#! /usr/bin/perl -w

# (c)1999 Ian Cass, Knowledge Matters Ltd.
# Read the GNU copyright stuff for all the legalese
#
# Check NTP time servers plugin. This plugin requires the ntpdate utility to
# be installed on the system, however since it's part of the ntp suite, you 
# should already have it installed.
#
# Nothing clever done in this program - its a very simple bare basics hack to
# get the job done.
#
# Things to do...
# check @words[9] for time differences greater than +/- x secs & return a
# warning.
#
# (c) 1999 Mark Jewiss, Knowledge Matters Limited
# 22-9-1999, 12:45
#
# Modified script to accept 2 parameters or set defaults.
# Now issues warning or critical alert is time difference is greater than the 
# time passed.
#
# These changes have not been tested completely due to the unavailability of a
# server with the incorrect time.
#
# (c) 1999 Bo Kersey, VirCIO - Managed Server Solutions <bo@vircio.com>
# 22-10-99, 12:17
#
# Modified the script to give useage if no parameters are input.
#
# Modified the script to check for negative as well as positive 
# time differences.
#
# Modified the script to work with ntpdate 3-5.93e Wed Apr 14 20:23:03 EDT 1999
#
# Modified the script to work with ntpdate's that return adjust or offset...
#
#
# Script modified 2000 June 01 by William Pietri <william@bianca.com>
#
# Modified script to handle weird cases:
#     o NTP server doesn't respond (e.g., has died)
#     o Server has correct time but isn't suitable synchronization
#           source. This happens while starting up and if contact
#           with master has been lost.
#


require 5.004;
use POSIX;
use strict;

sub usage;

my $TIMEOUT = 15;

my %ERRORS = ('UNKNOWN' , '-1',
	      'OK' , '0',
	      'WARNING', '1',
	      'CRITICAL', '2');

my $remote = shift || &usage(%ERRORS);
my $warning = shift || 60;
my $critical = shift || 120;

my $state = "OK";
my $answer = undef;
my $offset = undef;
my $msg; # first line of output to print if format is invalid

# Just in case of problems, let's not hang NetSaint
$SIG{'ALRM'} = sub {
     print ("ERROR: No response from ntp server (alarm)\n");
     exit $ERRORS{"UNKNOWN"};
};
alarm($TIMEOUT);

open (NTPDATE, "/usr/sbin/ntpdate -q $remote 2>&1 |");

while (<NTPDATE>) {
    $msg = $_ unless ($msg);
    if (/(offset|adjust)\s+(\-*\d+\.\d+)/i) {
	$offset = $2;
	last;
    }
}

# soak up remaining output; check for error
my $ntpdate_error = 0;
while (<NTPDATE>) { 
    $ntpdate_error = 1 if /no server suitable for synchronization found/;
};

# only declare an error if we also get a non-zero return code from ntpdate
$ntpdate_error = $ntpdate_error && !($?);

# An offset of 0.000000 with an error is probably bogus. Actually,
# it's probably always bogus, but let's be paranoid here.
if ($ntpdate_error && ($offset == 0)) { undef $offset;}

if (defined $offset) {
    if (abs($offset) > $warning) {
	if (abs($offset) > $critical) {
	    $state = "CRITICAL";
	    $answer = ": Time difference $offset seconds greater than +/- $critical sec\n";
	} else {
	    $state = "WARNING";
	    $answer = ": Time difference $offset seconds greater than +/- $warning sec\n";
	} 
    } else {
        if ($ntpdate_error) {
	    $state = "WARNING";
	    $answer = ": Time difference $offset seconds but server not suitable for synchronization\n";
        } else {
	    $state = "OK";
	    $answer = ": Time difference $offset seconds\n";
        }
    }
} else {
    if ($ntpdate_error) {
        $state = "CRITICAL";
        $answer = ": Server for ntp probably down\n";
    } else {
        $state = "UNKNOWN";
        $answer = ": Invalid format returned from ntpdate ($msg)\n";
    }
}
print ("$state$answer");
exit $ERRORS{$state};

sub usage {
	print "Minimum arguments not supplied!\n";
	print "\n";
	print "Perl Check NTP plugin for NetSaint\n";
	print "Copyright (c) 1999 Bo Kersey\n";
	print "\n";
	print "Usage: check_ntp.pl <host> [<warn> [<crit>]\n";
	print "\n";
	print "<warn> = Clock offset in seconds at which a warning message will be generated.\n	Defaults to 60.\n";
	print "<crit> = Clock offset in seconds at which a critical message will be generated.\n	Defaults to 120.\n\n";
	exit $ERRORS{"UNKNOWN"};
}









