/******************************************************************************
 *
 * CHECK_FPING.C
 *
 * Program: Fping plugin for NetSaint
 * License: GPL
 * Copyright (c) 1999 Didi Rieder (adrieder@sbox.tu-graz.ac.at)
 * $Date: 2000/06/04 15:42:03 $
 *
 * Modifications:
 *
 * 08-24-1999 Didi Rieder (adrieder@sbox.tu-graz.ac.at)
 *            Intial Coding
 * 09-11-1999 Karl DeBisschop (kdebiss@alum.mit.edu)
 *            Change to spopen
 *            Fix so that state unknown is returned by default
 *            (formerly would give state ok if no fping specified)
 *            Add server_name to output
 *            Reformat to 80-character standard screen
 * 11-18-1999 Karl DeBisschop (kdebiss@alum.mit.edu)
 *            set STATE_WARNING of stderr written or nonzero status returned
 *
 * Command line: CHECK_FPING <host_address> 
 *
 * Description:
 *
 * This plugin will use the /bin/fping command (form saint) to ping
 * the specified host for a fast check if the host is alive. Note that
 * it is necessary to set the suid flag on fping.
 *****************************************************************************/

#include "../common/config.h"
#include "../common/common.h"
#include "popen.h"
#include "utils.h"

int main(int argc, char **argv){
	char server_name[256];
	char command_line[256];
	char input_buffer[MAX_INPUT_BUFFER];
	int status=STATE_UNKNOWN;
	
	if(argc<2||argc>2){
		printf("Incorrect number of arguments supplied\n");
		printf("\n");
		print_revision(argv[0],"$Revision: 1.5 $");
		printf("Copyright (c) 1999 Didi Rieder (adrieder@sbox.tu-graz.ac.at)\n");
		printf("License: GPL\n");
		printf("\n");
		printf("Usage: %s <host_address>\n",argv[0]);
		printf("\n");
		printf("This plugin will use the /bin/fping command (from saint) to ping the specified\n");
		printf("host for a fast check if the host is alive. Note that it is necessary to set\n");
		printf("the suid flag on fping.\n");
		printf("\n");
		return STATE_UNKNOWN;
	        }

	/* get the server address */
	strcpy(server_name,argv[1]);

	/* run the command */
	sprintf(command_line,"%s %s",FPING_COMMAND,server_name);
	child_process=spopen(command_line);
	if(child_process==NULL){
		printf("Unable to open pipe: %s",command_line);
		return STATE_UNKNOWN;
	}

	child_stderr=fdopen(child_stderr_array[fileno(child_process)],"r");
	if(child_stderr==NULL){
		printf("Could not open stderr for %s\n",command_line);
	        }

	while(fgets(input_buffer,MAX_INPUT_BUFFER-1,child_process)){
		/* get the output */
		if(strstr(input_buffer,"not found")){
			printf("FPING unknown - %s not found\n",server_name);
			status=STATE_UNKNOWN;
			}
		else if(strstr(input_buffer,"is unreachable")){
			printf("FPING critical - %s is unreachable\n",server_name);
			status=STATE_CRITICAL;
			}
		else if(strstr(input_buffer,"is down")){
			printf("FPING critical - %s is down\n",server_name);
			status=STATE_CRITICAL;
			}
		else if(strstr(input_buffer,"is alive")){
			printf("FPING ok - %s is up\n",server_name);
			status=STATE_OK;
			}
		}

	/* If we get anything on STDERR, at least set warning */
	while(fgets(input_buffer,MAX_INPUT_BUFFER-1,child_stderr))
		status=max(status,STATE_WARNING);

	(void)fclose(child_stderr);

	/* close the pipe */
	if(spclose(child_process))
		status=max(status,STATE_WARNING);
	
	return status;
}
