/******************************************************************************
 *
 * CHECK_IMAP.C
 *
 * Program: IMAP4 plugin for NetSaint
 * License: GPL
 * Copyright (c) 1999 Tom Shields (tom.shields@basswood.com)
 *
 * Last Modified: $Date: 2000/01/28 20:03:56 $
 *
 * Command line: CHECK_IMAP <host_address> [port]
 *
 * Description:
 *
 * This plugin will attempt to open an IMAP connection with the host.
 * Successul connects return STATE_OK, refusals and timeouts return
 * STATE_CRITICAL, other errors return STATE_UNKNOWN.  Successful
 * connects, but incorrect reponse messages from the host result in
 * STATE_WARNING return values.
 *
 * Modifications:
 * 04-13-1999 Tom Shields
 *	Initial code
 * 08-18-1999 Ethan Galstad <netsaint@linuxbox.com>
 *	Modified code to work with common plugin functions, added socket timeout, string
 *      length checking
 * 09-19-1999 Ethan Galstad <netsaint@linuxbox.com>
 *	Changed expect string from "+OK" to "* OK" and default port to 143
 *****************************************************************************/

#include "../common/config.h"
#include "../common/common.h"
#include "netutils.h"
#include "utils.h"

#define IMAP_PORT	143
#define IMAP_EXPECT1    "* OK"
#define IMAP_QUIT	"a1 LOGOUT\n"


int server_port=IMAP_PORT;
char server_address[MAX_HOST_ADDRESS_LENGTH];


int main(int argc, char **argv){
	int sd;
	int result;
	char buffer[MAX_INPUT_BUFFER];

	if(argc<2||argc>4){
		printf("Incorrect number of arguments supplied\n");
		printf("\n");
		print_revision(argv[0],"$Revision: 1.4 $");
		printf("Copyright (c) 1999 Tom Shields (tom.shields@basswood.com)\n");
		printf("License: GPL\n");
		printf("\n");
		printf("Usage: %s <host_address> [port]\n",argv[0]);
		printf("\n");
		printf("This plugin will attempt to contact the IMAP4 service on the specified host.\n");
		printf("If possible, supply and IP address for the host address, as this will bypass the\n");
		printf("DNS lookup.  If you do not specify a port, the plugin will attempt to connect\n");
		printf("to the host on port %d.\n",server_port);
		printf("\n");
		return STATE_UNKNOWN;
	        }

	/* get the server address */
	strncpy(server_address,argv[1],sizeof(server_address)-1);
	server_address[sizeof(server_address)-1]=0;

	/* get the port number if one has been specified on the command line */
	if(argc>=3)
		server_port=atoi(argv[2]);

	/* initialize alarm signal handling */
	signal(SIGALRM,socket_timeout_alarm_handler);

	/* set socket timeout */
	alarm(socket_timeout);

	/* try to connect to the host at the given port number */
	time(&start_time);
	result=my_tcp_connect(server_address,server_port,&sd);

	/* we connected, so close connection before exiting */
	if(result==STATE_OK){

		/* watch for the IMAP connection string */
		result=recv(sd,buffer,MAX_INPUT_BUFFER-1,0);

		/* strip carriange returns */
		strip(buffer);

		/* return a WARNING status if we couldn't read any data */
		if(result==-1){
			printf("recv() failed\n");
			result=STATE_WARNING;
		        }

		else{

			/* make sure we find the response we are looking for */
			if(!strstr(buffer,IMAP_EXPECT1)){
				if(server_port==IMAP_PORT)
					printf("Invalid IMAP response received from host\n");
				else
					printf("Invalid IMAP response received from host on port %d\n",server_port);
				result=STATE_WARNING;
			        }

			else{
				time(&end_time);
				printf("IMAP ok - %d second response time\n",(int)(end_time-start_time));
				result=STATE_OK;
			        }
		        }

		/* close the connection */
		send(sd,IMAP_QUIT,strlen(IMAP_QUIT),0);
		close(sd);
	        }

	/* reset the alarm handler */
	alarm(0);

	return result;
        }
