/******************************************************************************
 *
 * CHECK_MYSQL.C
 *
 * Program: Mysql plugin for NetSaint
 * License: GPL
 * Copyright (c) 1999 Didi Rieder (adrieder@sbox.tu-graz.ac.at)
 * 
 * Last Modified: $Date: 2000/01/28 20:03:56 $
 *
 * Command line: check_mysql <host> <db_user> <db_pass> <db> <port>
 *
 * Description:
 *
 * This plugin is for testing a mysql server.
 *****************************************************************************/

#include "../common/common.h"
#include "utils.h"

#include <mysql/mysql.h>
#include <mysql/errmsg.h>


int main(int argc, char **argv) {

    MYSQL mysql;
    char result[1024];
    const char *db_user, *db_host, *db_pass, *db;
    unsigned int db_port;

    /* check if we got all parameters */
    if(argc != 6){
        printf("Incorrect number of arguments supplied\n");
	printf("\n");
	print_revision(argv[0],"$Revision: 1.4 $");
	printf("Copyright (c) 1999 Didi Rieder (adrieder@sbox.tu-graz.ac.at)\n");
        printf("License: GPL\n");
        printf("\n");
        printf("Usage: %s <host> <db_user> <db_pass> <db> <port>\n",argv[0]);
        printf("\n");
        return STATE_UNKNOWN;
    }


    /* get the parametes  */
    db_host = argv[1];
    db_user = argv[2];
    db_pass = argv[3];
    db      = argv[4];
    db_port = atoi(argv[5]);

    /* initialize mysql  */
    mysql_init(&mysql);

    /* establish a connection to the server and error checking*/
    if (!mysql_real_connect(&mysql,db_host,db_user,db_pass,db,db_port,NULL,0)){
	if(mysql_errno(&mysql) == CR_UNKNOWN_HOST) {
	    printf("%s\n", mysql_error(&mysql));
	    return STATE_WARNING;
	}
	if(mysql_errno(&mysql) == CR_VERSION_ERROR) {
	    printf("%s\n", mysql_error(&mysql));
	    return STATE_WARNING;
	}
	if(mysql_errno(&mysql) == CR_OUT_OF_MEMORY) {
	    printf("%s\n", mysql_error(&mysql));
	    return STATE_WARNING;
	}
	if(mysql_errno(&mysql) == CR_IPSOCK_ERROR) {
	    printf("%s\n", mysql_error(&mysql));
	    return STATE_WARNING;
	
	}
	if(mysql_errno(&mysql) == CR_SOCKET_CREATE_ERROR) {
	    printf("%s\n", mysql_error(&mysql));
	    return STATE_WARNING;
	}	
        printf("%s\n", mysql_error(&mysql));
	return STATE_CRITICAL;
     }

    /* get the server stats */
    sprintf(result,mysql_stat(&mysql));

    /* error checking once more */
    if(mysql_error(&mysql)) {
        if(mysql_errno(&mysql) == CR_SERVER_GONE_ERROR) {
            printf("%s\n", mysql_error(&mysql));
            return STATE_CRITICAL;
	}	
        if(mysql_errno(&mysql) == CR_SERVER_LOST) {
            printf("%s\n", mysql_error(&mysql));
	    return STATE_CRITICAL;
        }
	if(mysql_errno(&mysql) == CR_UNKNOWN_ERROR) {
    	    printf("%s\n", mysql_error(&mysql));
    	    return STATE_UNKNOWN;
	}
    }
    
    /* close the connection */		
    mysql_close(&mysql);

    /* print out the result of stats */
    printf("%s\n",result);

    return STATE_OK;    
}
