/******************************************************************************
 *
 * CHECK_NETSAINT.C
 *
 * Program: NetSaint process plugin for NetSaint
 * License: GPL
 * Copyright (c) 1999 Ethan Galstad (netsaint@linuxbox.com)
 *
 * Last Modified: $Date: 2000/05/03 03:11:03 $
 *
 * Command line: CHECK_NETSAINT
 *
 * Description:
 *
 *
 * License Information:
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *****************************************************************************/

#include "../common/common.h"
#include "popen.h"
#include "utils.h"

int main(int argc, char **argv){
	int result=STATE_UNKNOWN;
	char status_log[MAX_INPUT_BUFFER];
	char input_buffer[MAX_INPUT_BUFFER];
	char process_string[MAX_INPUT_BUFFER];
	int expire_minutes=0;
	unsigned long latest_entry_time=0L;
	unsigned long temp_entry_time=0L;
	int proc_entries=0;
	time_t current_time;
	char *temp_ptr;
	FILE *fp;

	if(argc!=4){
		printf("Incorrect number of arguments supplied\n");
		printf("\n");
		print_revision(argv[0],"$Revision: 1.5 $");
		printf("Copyright (c) 1999 Ethan Galstad (netsaint@linuxbox.com)\n");
		printf("License: GPL\n");
		printf("\n");
		printf("Usage: %s <status_log> <expire_minutes> <process_string>\n",argv[0]);
		printf("\n");
		printf("Notes:\n");
		printf("\n");
		printf("This plugin attempts to check the status of the NetSaint process on the local\n");
		printf("machine. The plugin will check to make sure the NetSaint status log is no older\n");
		printf("than the number of minutes specified by the <expire_minutes> option.  It also\n");
		printf("uses the /bin/ps command to check for a process matching whatever you specify\n");
		printf("by the <process_string> argument.\n");
		printf("\n");
		printf("Example:\n");
		printf("  ./check_netsaint /usr/local/netsaint/var/status.log 5 /usr/local/netsaint/bin/netsaint\n");  
		printf("\n");
		return STATE_UNKNOWN;
	        }

	/* get the name of the status log to check */
	strncpy(status_log,argv[1],sizeof(status_log));
	status_log[sizeof(status_log)-1]=0;

	/* get the number of minutes before the status log data is considered stale... */
	expire_minutes=atoi(argv[2]);

	/* get the process string to look for.. */
	strncpy(process_string,argv[3],sizeof(process_string));
	process_string[sizeof(process_string)-1]=0;

	/* Set signal handling and alarm */
	if(signal(SIGALRM,timeout_alarm_handler)==SIG_ERR){
		printf("Cannot catch SIGALRM");
		return STATE_UNKNOWN;
	        }

	/* handle timeouts gracefully... */
	alarm(timeout_interval);

	/* open the status log */
	fp=fopen(status_log,"r");
	if(fp==NULL){
		printf("Error: Cannot open status log for reading!\n");
		return STATE_CRITICAL;
	        }

	/* get the date/time of the last item updated in the log */
	while(fgets(input_buffer,MAX_INPUT_BUFFER-1,fp)){
		temp_ptr=strtok(input_buffer,"]");
		temp_entry_time=(temp_ptr==NULL)?0L:strtoul(temp_ptr+1,NULL,10);
		if(temp_entry_time>latest_entry_time)
			latest_entry_time=temp_entry_time;
	        }
	fclose(fp);

	/* run the command to check for the NetSaint process.. */
	child_process=spopen(PS_RAW_COMMAND);
	if(child_process==NULL){
		printf("Could not open pipe: %s\n",PS_RAW_COMMAND);
		return STATE_UNKNOWN;
	        }

	child_stderr=fdopen(child_stderr_array[fileno(child_process)],"r");
	if(child_stderr==NULL){
		printf("Could not open stderr for %s\n",PS_RAW_COMMAND);
	        }

	/* cound the number of matching NetSaint processes... */
	while(fgets(input_buffer,MAX_INPUT_BUFFER-1,child_process)){
		if(strstr(input_buffer,process_string))
			proc_entries++;
	        }
	/* If we get anything on stderr, at least set warning */
	while(fgets(input_buffer,MAX_INPUT_BUFFER-1,child_stderr))
		result=max(result,STATE_WARNING);

	/* close stderr */
	(void)fclose(child_stderr);

	/* close the pipe */
	if(spclose(child_process))
		result=max(result,STATE_WARNING);

	/* reset the alarm handler */
	alarm(0);

	if(proc_entries==0){
		printf("Could not locate a running NetSaint process!\n");
		return STATE_CRITICAL;
	        }

	result=STATE_OK;

	time(&current_time);
	if((current_time-latest_entry_time)>(expire_minutes*60))
		result=STATE_WARNING;

	printf("NetSaint %s: located %d process%s, status log updated %d second%s ago\n",(result==STATE_OK)?"ok":"problem",proc_entries,(proc_entries==1)?"":"es",(int)(current_time-latest_entry_time),((int)(current_time-latest_entry_time)==1)?"":"s");


	return result;
        }

